/** This is a random effect model for the coefficients but without the origin 
    and added var comps to weight for number of observations nobs in each 
    datapoint **/

data {
  int N;   //number of datapoints
  int K;   //number of shrunken predictors
  int M;   //total num predictors
  vector<lower=1>[N] nobs;  //number of observations within datapoints
                            //(coded as vector to avoid crash in rmarkdown)
  vector[N]    y;
  matrix[N,K]  preds;  //coefs shrunk first
  matrix[N,M-K] covs;  //covariates, not shrunk

  //informative priors for residuals
  real<lower=0> logsigmaloc;
  real<lower=0> logsigmascale;
}
parameters {
  vector[K] beta;  //random effect
  vector[M-K] theta;
  real<lower=0> sigmabeta;
  
  //residual variance model
  real logsigma_within;  //variance within datapoints
  real logsigma_betw;    //variance between datapoints
}
transformed parameters {
  //residual variance model
  real sigma_within;   //within
  real sigma_betw;     //between
  vector[N] disp;      //overall dispersion within + between
  real mndisp;
  
  //predictors
  vector[N] f;     //predictor
  vector[N] fadjust; //predicted adjustment
  
  //computations residual variance model
  sigma_within = exp(logsigma_within);
  sigma_betw   = exp(logsigma_betw);
  disp   = sqrt(sigma_within^2 ./ nobs + sigma_betw^2);
  mndisp = sqrt(mean(sigma_within^2 ./ nobs + sigma_betw^2));
  
  //linear predictor
  f = preds * beta + covs * theta;
  fadjust = preds * beta;
}
model {
  //linear model. Modelling betas with t or random makes little difference.
  //beta ~ student_t(1, 0, sigmabeta);
  beta ~ normal(0, sigmabeta);
  sigmabeta ~ cauchy(0, 1);
  
  //datapoint dispersion model
  logsigma_within ~ normal(logsigmaloc, logsigmascale);
  logsigma_betw   ~ normal(logsigmaloc, logsigmascale);

  //likelihood
  y ~ normal(f, disp);
}
